action: add const qualifier
[model-checker.git] / action.h
index 53107ea81762d33ab6a63ef2fa7e69176b379b1c..8ccebe395d5b0c7fdd2805fb0591e39e98b3ae65 100644 (file)
--- a/action.h
+++ b/action.h
@@ -1,3 +1,7 @@
+/** @file action.h
+ *  @brief Models actions taken by threads.
+ */
+
 #ifndef __ACTION_H__
 #define __ACTION_H__
 
@@ -21,36 +25,41 @@ typedef enum action_type {
 /* Forward declaration */
 class Node;
 class ClockVector;
+
 /**
  * The ModelAction class encapsulates an atomic action.
  */
-
 class ModelAction {
 public:
        ModelAction(action_type_t type, memory_order order, void *loc, int value);
        ~ModelAction();
-       void print(void);
+       void print(void) const;
 
-       thread_id_t get_tid() { return tid; }
-       action_type get_type() { return type; }
-       memory_order get_mo() { return order; }
-       void * get_location() { return location; }
+       thread_id_t get_tid() const { return tid; }
+       action_type get_type() const { return type; }
+       memory_order get_mo() const { return order; }
+       void * get_location() const { return location; }
        int get_seq_number() const { return seq_number; }
 
-       Node * get_node() { return node; }
+       Node * get_node() const { return node; }
        void set_node(Node *n) { node = n; }
 
-       bool is_read();
-       bool is_write();
-       bool is_acquire();
-       bool is_release();
-       bool same_var(ModelAction *act);
-       bool same_thread(ModelAction *act);
-       bool is_dependent(ModelAction *act);
+       bool is_read() const;
+       bool is_write() const;
+       bool is_rmw() const;
+       bool is_acquire() const;
+       bool is_release() const;
+       bool is_seqcst() const;
+       bool same_var(const ModelAction *act) const;
+       bool same_thread(const ModelAction *act) const;
+       bool is_synchronizing(const ModelAction *act) const;
 
        void create_cv(ModelAction *parent = NULL);
+       ClockVector * get_cv() const { return cv; }
        void read_from(ModelAction *act);
 
+       bool happens_before(ModelAction *act);
+
        inline bool operator <(const ModelAction& act) const {
                return get_seq_number() < act.get_seq_number();
        }
@@ -73,16 +82,18 @@ private:
        /** The thread id that performed this action. */
        thread_id_t tid;
        
/** The value written.  This should probably be something longer. */
        int value;
 
+       /** A back reference to a Node in NodeStack, if this ModelAction is
+        * saved on the NodeStack. */
        Node *node;
        
        int seq_number;
 
-       /** The clock vector stored with this action if this action is a
-        *  store release */
-
+       /** The clock vector stored with this action; only needed if this
+        * action is a store release? */
        ClockVector *cv;
 };
@@ -73,16 +82,18 @@ private:
        thread_id_t tid;
        
-       /** The value written.  This should probably be something longer. */
+       /** The value read or written (if RMW, then the value written). This
+        * should probably be something longer. */
        int value;
 